'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var reactQuery = require('@tanstack/react-query');

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

function formatData(event, queryClient) {
  var _a;
  var eventType = event.type;
  var queryKey = event.query.queryKey;
  var queryHash = event.query.queryHash;
  var timestamp = new Date(event.query.state.dataUpdatedAt);
  // JD: I think we can just delete this line
  if (queryHash === '["test-data"]') return;
  // handle scenarios where queryKey or queryHash or eventType is undefined (not sure when this occurs), but it was the reported bug
  if (!eventType || !queryKey || !queryHash) return;
  if (eventType === 'updated' && ((_a = event.action) === null || _a === void 0 ? void 0 : _a.type) === 'success') {
    var queryData = queryClient.getQueryData(queryKey);
    return {
      eventType: eventType,
      queryKey: queryKey,
      queryHash: queryHash,
      timestamp: timestamp,
      queryData: queryData
    };
  }
  if (eventType === 'removed') {
    return {
      eventType: eventType,
      queryKey: queryKey,
      queryHash: queryHash,
      timestamp: timestamp
    };
  }
  return null;
}

function Subscription(_a) {
  var _this = this;
  var handleMessages = _a.handleMessages;
  var queryClient = reactQuery.useQueryClient();
  var queryCache = queryClient.getQueryCache();
  var handleQueryCacheChange = function handleQueryCacheChange(event) {
    return __awaiter(_this, void 0, void 0, function () {
      var message;
      return __generator(this, function (_a) {
        message = formatData(event, queryClient);
        if (message) handleMessages(message);
        return [2 /*return*/];
      });
    });
  };

  React.useEffect(function () {
    var unsubscribe = queryCache.subscribe(handleQueryCacheChange);
    return function () {
      return unsubscribe();
    };
  }, []);
  return /*#__PURE__*/React.createElement(React.Fragment, null);
}

function TimeTravel() {
  var queryClient = reactQuery.useQueryClient();
  var handleUpdateUi = function handleUpdateUi(message) {
    var _a;
    if (((_a = message.data) === null || _a === void 0 ? void 0 : _a.type) === 'update-ui') {
      var currentQuery = message.data.payload;
      currentQuery.forEach(function (queryState) {
        if (queryState.queryData !== '') {
          queryClient.setQueryData([queryState.queryKey.slice(2, -2)], queryState.queryData);
        }
      });
    }
  };
  React.useEffect(function () {
    window.addEventListener('message', handleUpdateUi);
    return function () {
      window.removeEventListener('message', handleUpdateUi);
    };
  }, []);
  return /*#__PURE__*/React.createElement(React.Fragment, null);
}

const ErrorBoundaryContext = React.createContext(null);

const initialState = {
  didCatch: false,
  error: null
};
let ErrorBoundary$1 = class ErrorBoundary extends React.Component {
  constructor(props) {
    super(props);
    this.resetErrorBoundary = this.resetErrorBoundary.bind(this);
    this.state = initialState;
  }
  static getDerivedStateFromError(error) {
    return {
      didCatch: true,
      error
    };
  }
  resetErrorBoundary() {
    const {
      error
    } = this.state;
    if (error !== null) {
      var _this$props$onReset, _this$props;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      (_this$props$onReset = (_this$props = this.props).onReset) === null || _this$props$onReset === void 0 ? void 0 : _this$props$onReset.call(_this$props, {
        args,
        reason: "imperative-api"
      });
      this.setState(initialState);
    }
  }
  componentDidCatch(error, info) {
    var _this$props$onError, _this$props2;
    (_this$props$onError = (_this$props2 = this.props).onError) === null || _this$props$onError === void 0 ? void 0 : _this$props$onError.call(_this$props2, error, info);
  }
  componentDidUpdate(prevProps, prevState) {
    const {
      didCatch
    } = this.state;
    const {
      resetKeys
    } = this.props;

    // There's an edge case where if the thing that triggered the error happens to *also* be in the resetKeys array,
    // we'd end up resetting the error boundary immediately.
    // This would likely trigger a second error to be thrown.
    // So we make sure that we don't check the resetKeys on the first call of cDU after the error is set.

    if (didCatch && prevState.error !== null && hasArrayChanged(prevProps.resetKeys, resetKeys)) {
      var _this$props$onReset2, _this$props3;
      (_this$props$onReset2 = (_this$props3 = this.props).onReset) === null || _this$props$onReset2 === void 0 ? void 0 : _this$props$onReset2.call(_this$props3, {
        next: resetKeys,
        prev: prevProps.resetKeys,
        reason: "keys"
      });
      this.setState(initialState);
    }
  }
  render() {
    const {
      children,
      fallbackRender,
      FallbackComponent,
      fallback
    } = this.props;
    const {
      didCatch,
      error
    } = this.state;
    let childToRender = children;
    if (didCatch) {
      const props = {
        error,
        resetErrorBoundary: this.resetErrorBoundary
      };
      if (typeof fallbackRender === "function") {
        childToRender = fallbackRender(props);
      } else if (FallbackComponent) {
        childToRender = React.createElement(FallbackComponent, props);
      } else if (fallback === null || React.isValidElement(fallback)) {
        childToRender = fallback;
      } else {
        throw error;
      }
    }
    return React.createElement(ErrorBoundaryContext.Provider, {
      value: {
        didCatch,
        error,
        resetErrorBoundary: this.resetErrorBoundary
      }
    }, childToRender);
  }
};
function hasArrayChanged() {
  let a = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  let b = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return a.length !== b.length || a.some((item, index) => !Object.is(item, b[index]));
}

// Fallback component to display when an error is caught
var ErrorFallback = function ErrorFallback(_a) {
  var error = _a.error;
  console.error('ERROR IN REACT QUERY REWIND: ', error);
  console.log("%c Ensure the ReactQueryRewind component is inside query client. If the issue persists, please disable React Query Rewind and submit an issue on Github: https://github.com/oslabs-beta/react-query-rewind", 'color: red; background-color: lightyellow; font-weight: bold; padding: 2px;');
  return /*#__PURE__*/React.createElement(React.Fragment, null);
};
// ErrorBoundary component wrapping the ErrorFallback
var ErrorBoundary = function ErrorBoundary(_a) {
  var children = _a.children;
  return /*#__PURE__*/React.createElement(ErrorBoundary$1, {
    FallbackComponent: ErrorFallback
  }, children);
};

function ReactQueryRewind() {
  var _a = React.useState(false),
    timeTravel = _a[0],
    setTimeTravel = _a[1];
  var contentConnectedRef = React.useRef(false);
  var contentMessageQueueRef = React.useRef([]);
  var sendContentMessageQueue = function sendContentMessageQueue() {
    contentMessageQueueRef.current.forEach(sendEvent);
    contentMessageQueueRef.current = [];
  };
  var handleMessages = function handleMessages(message) {
    if (contentConnectedRef.current) {
      sendEvent(message);
    } else {
      contentMessageQueueRef.current.push(message);
    }
  };
  var sendEvent = function sendEvent(event) {
    window.postMessage({
      framework: 'react',
      type: 'event',
      payload: event
    }, '*');
  };
  var handleContentMessages = function handleContentMessages(message) {
    var _a, _b;
    if (((_a = message.data) === null || _a === void 0 ? void 0 : _a.type) === 'content-script-ready') {
      contentConnectedRef.current = true;
      window.postMessage({
        type: 'app-connected'
      }, '*');
      sendContentMessageQueue();
    }
    if (((_b = message.data) === null || _b === void 0 ? void 0 : _b.type) === 'time-travel') {
      setTimeTravel(message.data.payload);
    }
  };
  React.useEffect(function () {
    window.addEventListener('message', handleContentMessages);
    return function () {
      window.removeEventListener('message', handleContentMessages);
    };
  }, []);
  return /*#__PURE__*/React.createElement(ErrorBoundary, null, timeTravel ? ( /*#__PURE__*/React.createElement(TimeTravel, null)) : ( /*#__PURE__*/React.createElement(Subscription, {
    handleMessages: handleMessages
  })));
}

exports.default = ReactQueryRewind;
//# sourceMappingURL=index.cjs.map
